# ---  clear workspace 
rm(list = ls())

# ---------------------------------------------------------------------------------

# ---- SETTNGS  ----

# --- Run Data Preparation   - execute only once (to generate ./data/PrepData.rds) 

# source ("0_extract_RawData.R")

# --- load data list (d)

load ( file = ".\\data\\PrepData.rds")  


# ---------------------------------------------------------------------------------



# ---- define functionality levels ----

Fun_level = c( 1,  # Shelter
               1,  # Access
               1,  # Life Support
               1,  # Export for Shelter
               1,  # Export for Access
               1,  # Export for Life Support
               1)  # Other Life Support ( Statistcial Differences, Inventory and Value change)

for( zz in 1:7){
  
  Fun_level = Fun_level * 0
  Fun_level[zz] = 1

  print(paste0("RUN ", zz))
  # ---------------------------------------------------------------------------------
  # -------------------------- Input - Output Analysis --------------------------------
  # ---------------------------------------------------------------------------------
  
  
  final_demand = t( t( d$Fun_structure) * d$Fun_base * Fun_level)  # total final demand matrix
  
  final_demand_dom = rowSums( final_demand * ( 1- d$mFun ) )
  final_demand_imp = rowSums( final_demand * (    d$mFun ) )
  
  
  # --- Part 1 --- Indirect Produciton
  
  q.1 =  solve ( d$I - d$D %*% (d$Ut - d$Um)) %*% d$D %*%  final_demand_dom
  USE.1  = t( t( d$Ut ) * as.vector (q.1) ) 
  
  USE_dom.1 = t( t( d$Ut - d$Um ) * as.vector (q.1) ) 
  SUP.1     = t( d$D) * (rowSums( USE_dom.1) + final_demand_dom ) 
  
  # --- Part 2 --- Investment Production
  
  investments            =  (t( t( d$Inv_structure) *  as.vector (q.1  * d$Inv_per_Q) ) )  # investment total
  investments_demand_dom = rowSums( investments * (1-d$mInv) )    
  
  q.2 = solve ( d$I - d$D %*% (d$Ut - d$Um)) %*% d$D %*%  investments_demand_dom
  
  USE.2  = t( t( d$Ut ) * as.vector (q.2) ) 
  USE_dom.2 = t( t( d$Ut - d$Um ) * as.vector (q.2) ) 
  SUP.2     = t( d$D) * (rowSums( USE_dom.2) + investments_demand_dom )
  
  
  # --- Part 1+2 -- ALL
  
  # q       = q.1+q.2
  # USE     = USE.1 + USE.2
  # USE_dom = USE_dom.1 + USE_dom.2
  # SUP     = SUP.1+SUP.2
  
  q       = q.1 
  USE     = USE.1
  USE_dom = USE_dom.1
  SUP     = SUP.1
  
  
  # --- Derive Primary Energy --- 
  
  Prim_Energy                    = matrix(0, ncol = 4,nrow = 12)
  rownames(Prim_Energy)          = c("Coal", "Coke","CrudeOil", "OilProd","CrudeGas","Waste","BioSolid","BioGaseous","BioLiquid","AmbientHeat","Electricity_Trade","Hydro_Wind_PV" )
  colnames (Prim_Energy)         = c("Indirect in units.1", "Investment in units.2", "Indirect in Terajoule.1", "Investment in Terajoule.2")
  
  for ( rnd in 1:2){
    
    if(rnd == 1 ) {
      q       = q.1 
      USE     = USE.1
      USE_dom = USE_dom.1
      SUP     = SUP.1
      
    } else {
      q       = q.2 
      USE     = USE.2
      USE_dom = USE_dom.2
      SUP     = SUP.2
    }
    
    # --------------------------
    
    Prim_Energy[1,rnd] = sum(USE[24,])                     # use of the product CPA05
    Prim_Energy[2,rnd] = sum(USE[26,]) - sum(SUP[26,])     # use of the product CPA19A minus Cokery output (supply of CPA19A)
    Prim_Energy[3,rnd] = sum(USE[22,])                     # use of the product CPA06A                     # use of the product CPA05
    Prim_Energy[4,rnd] = sum(USE[25,]) - sum(SUP[25,])     # use of the product CPA19B (oilproducts) minus Refinery output (supply of CPA19B)
    Prim_Energy[5,rnd] = sum(USE[23,])                     # use of product CPA06B (natural Gas)
    Prim_Energy[6,rnd] = -sum(USE[38,19])                  # total use of the sideproduct "Waste"
    Prim_Energy[7,rnd] = sum(USE[33,30:66])                # Transformation and Final Demand input of solid bio
    Prim_Energy[8,rnd] = -sum(USE[35,19])                  # use in transformation  (negative because BioGas is a sideproduct) 
    Prim_Energy[9,rnd] = sum(USE[34,-9] )                 # Transformation and Final Demand input of liquid bio  minus BioRefinery own demand
    Prim_Energy[10,rnd]= sum(USE[64,])+  final_demand_dom[64]                # alternativ:sum(USE[183:188,]) + sum(FinDem_dom[183:188])  ;  Just sum of final demand of ambient heat due to 100% efficiency
    
    # electricity
    
    el_dem = sum(USE[27,])              # Electricity Demand minus Electricity Production in Auxiliary sectors including Losses and OwnDemand
    el_aux = sum( USE[27,][USE[27,]<0] ) # Electricity from Auxiliary Sectors
    el_sup = SUP[27,27]
    
    Prim_Energy[11,rnd] = el_dem - el_sup 
    Prim_Energy[12,rnd] = el_sup * 0.7800404081
    
    # --- Derive Primary Energy (Energy units) --- 
    
    unit_to_TJ = c(0.0282708577773849 , 0.029006614243774 , 0.042457024970413 , 0.0422010290341717 , 0.0364176 , 0.0124457957197435 , 0.0116456561953133 , 0.0153806482852565 , 0.0140700521713321 , 1 , 1, 1)
    
    for ( i in 1:12){   Prim_Energy[i,rnd+2] = Prim_Energy[i,rnd] * unit_to_TJ[i] }
    
    
    # --- Derive Final Energy (Energy units) --- 
    
    Final_Energy = matrix( 0, ncol = 2, nrow = 26  )
    
    colnames(Final_Energy) = c( " Indirect", "Investment" )
    
    rownames(Final_Energy) = c(
     "Coal for Heating",
     "Coal for Motor",
     "Coke for Heating",
     "Coke for Motor",
     "Oilproducts for Heating",
     "Oilproducts for Motor",
     "Natural Gas for Heating",
     "Natural Gas for Motor",
     "Electricity for Heating",
     "Electricity for Motor",
     "District Heat for Heating",
     "District Heat for Motor",
     "Blast_Furnace Gas for Heating",
     "Blast_Furnace Gas for Motor",
     "Cokery Gas for Heating",
     "Cokery Gas for Motor",
     "Combustable Waste for Heating",
     "Combustable Waste for Motor",
     "Biomass Solid for Heating",
     "Biomass Solid for Motor",
     "Biomass Liquid for Heating",
     "Biomass Liquid for Motor",
     "Biomass Gaseous for Heating",
     "Biomass Gaseous for Motor",
     "Ambient Heat for Heating",
     "Electricity for IT/Lightning")
      
      
    Final_Energy[,1]  = rowSums(USE.1[40:65,]) + final_demand_dom[40:65]
    Final_Energy[,2]  = rowSums(USE.2[40:65,])  
    
    sum(Final_Energy)
  }
  
  
  
  sn = sum(Fun_level*1:7)
  
  xlsx::write.xlsx2(Prim_Energy, file = "Primary_Energy.xlsx", sheetName = paste0("Fun_",sn) , append = T)
  xlsx::write.xlsx2(Final_Energy, file = "Final_Energy.xlsx", sheetName = paste0("Fun_",sn) , append = T)

}



